--- src/client/misc.cpp.orig	Sat Nov 16 03:20:58 2002
+++ src/client/misc.cpp	Thu Jan 22 03:48:37 2004
@@ -32,6 +32,9 @@
 
 #ifdef HAVE_MNTENT_H
   #include <mntent.h>
+#elif defined(OS_FBSD)
+  #include <sys/disklabel.h>
+  #undef FS_HPFS
 #else
   #warning "mntent.h not found"
 #endif
@@ -293,6 +296,7 @@
 	nRes = fwrite(mbrOriginal[dwOriginalMbrNb].cData+MBR_SIZE_BOOT, MBR_SIZE_TABLE+2, 1, fDevice); // table + 0xAA55
     }
 
+#if defined(LINUX_OS)
   // re-read partition table
   if (ioctl(fileno(fDevice), BLKRRPART))
     {
@@ -303,6 +307,7 @@
     {
       showDebug(1, "Partition table was reread\n");
     }
+#endif
 
   // close and check error
   flock(fileno(fDevice), LOCK_UN);
@@ -1303,7 +1308,34 @@
   
   return bMounted;
 }
-#else // OS_LINUX
+#elif defined(OS_FBSD)
+int isMounted(const char *szDevice, char *szMountPoint)
+{
+  int i, mntsize;
+  struct statfs *mntbuf;
+  char szBuf [MAXPATHLEN+1];
+  const char *ptrDevice;
+
+  ptrDevice = szDevice;
+  if (realpath(ptrDevice, szBuf) != NULL)
+    ptrDevice = szBuf;
+
+  showDebug(3, "isMounted(%s) = isMounted(%s)\n", szDevice, ptrDevice);
+
+  mntsize = getmntinfo(&mntbuf, MNT_NOWAIT);
+  for (i = 0; i < mntsize; i++)
+    {
+      if (strcmp(ptrDevice, mntbuf[i].f_mntfromname) == 0)
+        {
+	  strcpy(szMountPoint, mntbuf[i].f_mntfromname);
+	  showDebug(3, "partition [%s] mounted on [%s]\n",
+	      mntbuf[i].f_mntfromname, szMountPoint);
+	}
+    }
+
+  return false;
+}
+#else // !OS_LINUX && !OS_FBSD
 int isMounted(const char *szDevice, char *szMountPoint)
 {
   #warning isMounted always return false
@@ -1332,7 +1364,11 @@
 QWORD getPartitionSize(const char *szDevice)
 {
   QWORD qwSize;
+#if defined(OS_LINUX)
   long lSize;
+#elif defined(OS_FBSD)
+  struct disklabel dl;
+#endif
   int fdDevice;
   
   qwSize = 0;
@@ -1344,8 +1380,10 @@
 #ifdef OS_LINUX 
   if (ioctl(fdDevice, BLKGETSIZE, &lSize) >= 0)
     qwSize = ((QWORD) 512) * ((QWORD) lSize);
-#endif
-#ifdef OS_FBSD
+#elif defined(OS_FBSD)
+  if (ioctl (fdDevice, DIOCGDINFO, &dl))
+      qwSize = dl.d_secperunit;
+#else
   #warning qwSize not initialized
 #endif
   
