--- src/shared/image_disk.cpp.orig	Sat May 11 04:52:47 2002
+++ src/shared/image_disk.cpp	Thu Jan 22 03:28:20 2004
@@ -36,6 +36,10 @@
 #ifdef HAVE_SYS_MOUNT_H
   #include <sys/mount.h>
 #endif
+#ifdef OS_FBSD
+  #include <sys/disklabel.h>
+  #undef FS_HPFS
+#endif
 
 #include <fcntl.h>
 #include <stdlib.h>
@@ -53,6 +57,9 @@
 
 #define SPACEFILE_SIZE (10*1024*1024) // 10 MB
 
+#ifdef OS_FBSD
+#define O_LARGEFILE 0
+#else
 #ifndef HAVE_FSTAT64
   #warning "fstat32 used"
 #endif
@@ -62,6 +69,7 @@
 #ifndef HAVE_OPEN64
   #warning "open32 used"
 #endif
+#endif
 
 // =======================================================
 CImageDisk::CImageDisk(COptions * options):CImageBase() // [Main-Thread]
@@ -85,8 +93,12 @@
 QWORD CImageDisk::getPartitionSize(const char *szDevice) // [Main-Thread]
 {
   QWORD qwSize;
-  long lSize;
   int fdDevice;
+#ifdef OS_FBSD
+  struct disklabel dl;
+#else
+  long lSize;
+#endif
 
   showDebug(1, "begin\n");
   qwSize = 0;
@@ -107,7 +119,14 @@
   else
     THROW( ERR_ERRNO, errno);
 #endif
-
+#ifdef OS_FBSD
+  if (!ioctl (fdDevice, DIOCGDINFO, &dl))
+    {
+      qwSize = dl.d_secperunit;
+    }
+  else
+    THROW( ERR_ERRNO, errno);
+#endif
   if (::close(fdDevice))
     THROW( ERR_ERRNO, errno);
 
@@ -394,15 +413,10 @@
   
       showDebug(1, "openReading[%s]\n", m_szImageFilename);
 
-    #ifdef OS_LINUX
-      #ifdef HAVE_OPEN64
-        m_nImageFd = open64(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
-      #else
-        m_nImageFd = open(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
-      #endif
-    #endif
-    #ifdef OS_FBSD
-      m_nImageFd = open(m_szImageFilename, O_RDONLY | O_NOFOLLOW);
+    #ifdef HAVE_OPEN64
+      m_nImageFd = open64(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
+    #else
+      m_nImageFd = open(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
     #endif
       if (m_nImageFd == -1)
         THROW(ERR_ERRNO, errno);
@@ -639,7 +653,7 @@
 
 // =======================================================
 // may throw errno
-#ifdef HAVE_SYS_STATFS_H
+#if defined(HAVE_SYS_STATFS_H) || defined(OS_FBSD)
 void CImageDisk::getFreeSpace(QWORD *qwAvailDiskSpace) // [Main-Thread]
 {
   int nRes;
@@ -720,7 +734,7 @@
 }
 
 // =======================================================
-#ifdef HAVE_SYS_STATFS_H
+#if defined(HAVE_SYS_STATFS_H) || defined(OS_FBSD)
 void CImageDisk::getDiskFreeSpaceForFile(QWORD *qwAvailDiskSpace, char *szFilepath) // [Main-Thread]
 {
   //::getDiskFreeSpaceForFile(qwFreeSpace, szFilepath);
